from datetime import _TzInfo, datetime, timedelta
from typing import TypeVar

_DateTimeT = TypeVar("_DateTimeT", bound=datetime)

def today(tzinfo: _TzInfo | None = None) -> datetime: ...
def default_tzinfo(dt: _DateTimeT, tzinfo: _TzInfo) -> _DateTimeT: ...
def within_delta(dt1: datetime, dt2: datetime, delta: timedelta) -> bool: ...
